@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / BIFS codec sub-project
@@ -10,35 +11,257 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
-
+#include <gpac/maths.h>
 #include <gpac/internal/bifs_dev.h>
-#include "quant.h" 
+#include "quant.h"
+
+#ifndef GPAC_DISABLE_BIFS
+
 
 GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field);
 GF_Err BD_DecMFFieldVec(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field);
 
 
 
-void gf_bifs_dec_name(GF_BitStream *bs, char *name)
+void gf_bifs_dec_name(GF_BitStream *bs, char *name, u32 size)
 {
+	Bool error = GF_FALSE;
 	u32 i = 0;
 	while (1) {
-		name[i] = gf_bs_read_int(bs, 8);
-		if (!name[i]) break;
+		char c = gf_bs_read_int(bs, 8);
+		if (i<size)
+			name[i] = c;
+		else {
+			error = GF_TRUE;
+		}
+		if (!c) break;
 		i++;
 	}
+	if (error) {
+		name[size-1] = 0;
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[BIFS] name too long %d bytes but max size %d, truncating\n", i, size));
+	}
+}
+
+
+static GF_Err BD_XReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
+{
+	GF_FieldInfo targetField, fromField;
+	GF_Node *target, *n, *fromNode;
+	s32 pos = -2;
+	void *slot_ptr;
+	u32 id, nbBits, ind, aind;
+	GF_Err e;
+
+
+	id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+	target = gf_sg_find_node(codec->current_graph, id);
+	if (!target) return GF_SG_UNKNOWN_NODE;
+
+	nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(target, GF_SG_FIELD_CODING_IN)-1);
+	ind = gf_bs_read_int(bs, nbBits);
+	e = gf_bifs_get_field_index(target, ind, GF_SG_FIELD_CODING_IN, &aind);
+	if (e) return e;
+	e = gf_node_get_field(target, aind, &targetField);
+	if (e) return e;
+
+	if (!gf_sg_vrml_is_sf_field(targetField.fieldType)) {
+		/*this is indexed replacement*/
+		if (gf_bs_read_int(bs, 1)) {
+			/*index is dynamic*/
+			if (gf_bs_read_int(bs, 1)) {
+				id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+				n = gf_sg_find_node(codec->current_graph, id);
+				if (!n) return GF_SG_UNKNOWN_NODE;
+
+				nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(n, GF_SG_FIELD_CODING_DEF)-1);
+				ind = gf_bs_read_int(bs, nbBits);
+				e = gf_bifs_get_field_index(n, ind, GF_SG_FIELD_CODING_DEF, &aind);
+				if (e) return e;
+				e = gf_node_get_field(n, aind, &fromField);
+				if (e) return e;
+
+				pos = 0;
+				switch (fromField.fieldType) {
+				case GF_SG_VRML_SFBOOL:
+					if (*(SFBool*)fromField.far_ptr) pos = 1;
+					break;
+				case GF_SG_VRML_SFINT32:
+					if (*(SFInt32*)fromField.far_ptr >=0) pos = *(SFInt32*)fromField.far_ptr;
+					break;
+				case GF_SG_VRML_SFFLOAT:
+					if ( (*(SFFloat *)fromField.far_ptr) >=0) pos = (s32) floor( FIX2FLT(*(SFFloat*)fromField.far_ptr) );
+					break;
+				case GF_SG_VRML_SFTIME:
+					if ( (*(SFTime *)fromField.far_ptr) >=0) pos = (s32) floor( (*(SFTime *)fromField.far_ptr) );
+					break;
+				}
+			} else {
+				u32 type = gf_bs_read_int(bs, 2);
+				switch (type) {
+				case 0:
+					pos = gf_bs_read_int(bs, 16);
+					break;
+				case 2:
+					pos = 0;
+					break;
+				case 3:
+					pos = -1;
+					break;
+				}
+			}
+		}
+
+		if (targetField.fieldType==GF_SG_VRML_MFNODE) {
+			if (gf_bs_read_int(bs, 1)) {
+				target = gf_node_list_get_child(*(GF_ChildNodeItem **)targetField.far_ptr, pos);
+				if (!target) return GF_SG_UNKNOWN_NODE;
+
+				nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(target, GF_SG_FIELD_CODING_IN)-1);
+				ind = gf_bs_read_int(bs, nbBits);
+				e = gf_bifs_get_field_index(target, ind, GF_SG_FIELD_CODING_IN, &aind);
+				if (e) return e;
+				e = gf_node_get_field(target, aind, &targetField);
+				if (e) return e;
+				pos = -2;
+			}
+		}
+	}
+
+	fromNode = NULL;
+	if (gf_bs_read_int(bs, 1)) {
+		id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+		fromNode = gf_sg_find_node(codec->current_graph, id);
+		if (!fromNode) return GF_SG_UNKNOWN_NODE;
+
+		nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(fromNode, GF_SG_FIELD_CODING_DEF)-1);
+		ind = gf_bs_read_int(bs, nbBits);
+		e = gf_bifs_get_field_index(fromNode, ind, GF_SG_FIELD_CODING_DEF, &aind);
+		if (e) return e;
+		e = gf_node_get_field(fromNode, aind, &fromField);
+		if (e) return e;
+	}
+
+	/*indexed replacement*/
+	if (pos>=-1) {
+		/*if MFNode remove the child and set new node*/
+		if (targetField.fieldType == GF_SG_VRML_MFNODE) {
+			GF_Node *newnode;
+			if (fromNode) {
+				newnode = *(GF_Node**)fromField.far_ptr;
+			} else {
+				newnode = gf_bifs_dec_node(codec, bs, targetField.NDTtype);
+			}
+			gf_node_replace_child(target, (GF_ChildNodeItem**) targetField.far_ptr, pos, newnode);
+			if (newnode) gf_node_register(newnode, NULL);
+		}
+		/*erase the field item*/
+		else {
+			u32 sftype;
+			if ((pos < 0) || ((u32) pos >= ((GenMFField *) targetField.far_ptr)->count) ) {
+				pos = ((GenMFField *)targetField.far_ptr)->count - 1;
+				/*may happen with text and default value*/
+				if (pos < 0) {
+					pos = 0;
+					gf_sg_vrml_mf_alloc(targetField.far_ptr, targetField.fieldType, 1);
+				}
+			}
+			e = gf_sg_vrml_mf_get_item(targetField.far_ptr, targetField.fieldType, & slot_ptr, pos);
+			if (e) return e;
+			sftype = gf_sg_vrml_get_sf_type(targetField.fieldType);
+
+			if (fromNode) {
+				if (sftype==fromField.fieldType)
+					gf_sg_vrml_field_copy(slot_ptr, fromField.far_ptr, sftype);
+			} else {
+				GF_FieldInfo sffield;
+				memcpy(&sffield, &targetField, sizeof(GF_FieldInfo));
+				sffield.fieldType = sftype;
+				sffield.far_ptr = slot_ptr;
+				gf_bifs_dec_sf_field(codec, bs, target, &sffield, GF_FALSE);
+			}
+		}
+		gf_bifs_check_field_change(target, &targetField);
+		return GF_OK;
+	}
+	switch (targetField.fieldType) {
+	case GF_SG_VRML_SFNODE:
+	{
+		GF_Node *newnode;
+		if (fromNode) {
+			newnode = *(GF_Node**)fromField.far_ptr;
+		} else {
+			newnode = gf_bifs_dec_node(codec, bs, targetField.NDTtype);
+		}
+
+		n = *((GF_Node **) targetField.far_ptr);
+
+		*((GF_Node **) targetField.far_ptr) = newnode;
+		if (newnode) gf_node_register(newnode, target);
+
+		if (n) gf_node_unregister(n, target);
+
+		break;
+	}
+	case GF_SG_VRML_MFNODE:
+	{
+		GF_ChildNodeItem *previous = * (GF_ChildNodeItem **) targetField.far_ptr;
+		* ((GF_ChildNodeItem **) targetField.far_ptr) = NULL;
+
+		if (fromNode) {
+			GF_ChildNodeItem *list, *prev, *cur;
+			list = * ((GF_ChildNodeItem **) targetField.far_ptr);
+			prev=NULL;
+			while (list) {
+				cur = (GF_ChildNodeItem*)gf_malloc(sizeof(GF_ChildNodeItem));
+				cur->next = NULL;
+				cur->node = list->node;
+				if (prev) {
+					prev->next = cur;
+				} else {
+					* ((GF_ChildNodeItem **) targetField.far_ptr) = cur;
+				}
+				gf_node_register(list->node, target);
+				prev = cur;
+				list = list->next;
+			}
+		} else {
+			e = gf_bifs_dec_field(codec, bs, target, &targetField, GF_FALSE);
+			if (e) return e;
+		}
+		if (previous)
+			gf_node_unregister_children(target, previous);
+
+		break;
+	}
+	default:
+		if (!gf_sg_vrml_is_sf_field(targetField.fieldType)) {
+			e = gf_sg_vrml_mf_reset(targetField.far_ptr, targetField.fieldType);
+		}
+		if (e) return e;
+
+		if (fromNode) {
+			if (fromField.fieldType == targetField.fieldType)
+				gf_sg_vrml_field_clone(targetField.far_ptr, fromField.far_ptr, targetField.fieldType, codec->current_graph);
+		} else {
+			e = gf_bifs_dec_field(codec, bs, target, &targetField, GF_FALSE);
+		}
+		break;
+	}
+	gf_bifs_check_field_change(target, &targetField);
+	return e;
 }
 
 static GF_Err BD_DecProtoDelete(GF_BifsDecoder * codec, GF_BitStream *bs)
@@ -72,10 +295,10 @@ static GF_Err BD_DecProtoDelete(GF_BifsDecoder * codec, GF_BitStream *bs)
 static GF_Err BD_DecMultipleIndexReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
 	u32 ID, ind, field_ind, NumBits, lenpos, lennum, count, pos;
-	GF_Node *node, *new_node;
+	GF_Node *node;
 	GF_Err e;
 	GF_FieldInfo field, sffield;
-	
+
 	ID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, ID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
@@ -94,7 +317,8 @@ static GF_Err BD_DecMultipleIndexReplace(GF_BifsDecoder * codec, GF_BitStream *b
 	/*cf index value replace */
 	if (field.fieldType == GF_SG_VRML_MFNODE) {
 		while (count) {
-			pos = gf_bs_read_int(bs, lenpos);		
+			GF_Node *new_node;
+			pos = gf_bs_read_int(bs, lenpos);
 			/*first decode*/
 			new_node = gf_bifs_dec_node(codec, bs, field.NDTtype);
 			if (!new_node) return codec->LastError;
@@ -120,7 +344,7 @@ static GF_Err BD_DecMultipleIndexReplace(GF_BifsDecoder * codec, GF_BitStream *b
 
 		e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, & sffield.far_ptr, pos);
 		if (e) return e;
-		e = gf_bifs_dec_sf_field(codec, bs, node, &sffield);
+		e = gf_bifs_dec_sf_field(codec, bs, node, &sffield, GF_FALSE);
 		if (e) break;
 		count--;
 	}
@@ -137,12 +361,12 @@ static GF_Err BD_DecMultipleReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
-	
+
 	flag = gf_bs_read_int(bs, 1);
 	if (flag) {
-		e = gf_bifs_dec_node_mask(codec, bs, node, 0);
+		e = gf_bifs_dec_node_mask(codec, bs, node, GF_FALSE);
 	} else {
-		e = gf_bifs_dec_node_list(codec, bs, node, 0);
+		e = gf_bifs_dec_node_list(codec, bs, node, GF_FALSE);
 	}
 	return e;
 }
@@ -159,7 +383,7 @@ static GF_Err BD_DecGlobalQuantizer(GF_BifsDecoder * codec, GF_BitStream *bs)
 		codec->scenegraph->global_qp = NULL;
 	}
 	codec->ActiveQP = NULL;
-	
+
 	if (!node || (gf_node_get_tag(node) != TAG_MPEG4_QuantizationParameter)) {
 		if (node) gf_node_unregister(node, NULL);
 		return codec->LastError;
@@ -180,9 +404,10 @@ static GF_Err BD_DecNodeDeleteEx(GF_BifsDecoder * codec, GF_BitStream *bs)
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
-	return gf_node_replace(node, NULL, 1);
+	return gf_node_replace(node, NULL, GF_TRUE);
 }
 
+#ifdef GPAC_UNUSED_FUNC
 static GF_Err BD_DecOperandReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
 	s32 pos;
@@ -224,7 +449,7 @@ static GF_Err BD_DecOperandReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 		case 3:
 			/*-1 means append*/
 			pos = ((GenMFField *)field.far_ptr)->count;
-			if (!pos) 
+			if (!pos)
 				return GF_NON_COMPLIANT_BITSTREAM;
 			pos -= 1;
 			break;
@@ -269,7 +494,7 @@ static GF_Err BD_DecOperandReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 		case 3:
 			/*-1 means append*/
 			pos = ((GenMFField *)src_field.far_ptr)->count;
-			if (!pos) 
+			if (!pos)
 				return GF_NON_COMPLIANT_BITSTREAM;
 			pos -= 1;
 			break;
@@ -286,6 +511,7 @@ static GF_Err BD_DecOperandReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	gf_sg_vrml_field_copy(dst_ptr, src_ptr, src_type);
 	return GF_OK;
 }
+#endif /*GPAC_UNUSED_FUNC*/
 
 static GF_Err BD_DecExtendedUpdate(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
@@ -308,7 +534,7 @@ static GF_Err BD_DecExtendedUpdate(GF_BifsDecoder * codec, GF_BitStream *bs)
 	case 6:
 		return BD_DecNodeDeleteEx(codec, bs);
 	case 7:
-		return BD_DecOperandReplace(codec, bs);
+		return BD_XReplace(codec, bs);
 	default:
 		return GF_BIFS_UNKNOWN_VERSION;
 	}
@@ -369,7 +595,7 @@ static GF_Err BD_DecIndexInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 	u32 NumBits, ind, field_ind;
 	u8 type;
 	s32 pos;
-	GF_Node *def, *node;
+	GF_Node *def;
 	GF_FieldInfo field, sffield;
 
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -406,7 +632,7 @@ static GF_Err BD_DecIndexInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 
 	/*rescale the MFField and parse the SFField*/
 	if (field.fieldType==GF_SG_VRML_MFNODE) {
-		node = gf_bifs_dec_node(codec, bs, field.NDTtype);
+		GF_Node *node = gf_bifs_dec_node(codec, bs, field.NDTtype);
 		if (!node) return codec->LastError;
 
 		e = gf_node_register(node, def);
@@ -417,7 +643,7 @@ static GF_Err BD_DecIndexInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 		} else {
 			e = gf_node_list_insert_child((GF_ChildNodeItem **) field.far_ptr, node, pos);
 		}
-		
+
 		if (!e) gf_bifs_check_field_change(def, &field);
 	} else {
 		if (pos == -1) {
@@ -427,7 +653,7 @@ static GF_Err BD_DecIndexInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 			e = gf_sg_vrml_mf_insert(field.far_ptr, field.fieldType, & sffield.far_ptr, pos);
 		}
 		if (e) return e;
-		e = gf_bifs_dec_sf_field(codec, bs, def, &sffield);
+		e = gf_bifs_dec_sf_field(codec, bs, def, &sffield, GF_FALSE);
 		if (!e) gf_bifs_check_field_change(def, &field);
 	}
 	return e;
@@ -448,7 +674,7 @@ static GF_Err BD_DecInsert(GF_BifsDecoder * codec, GF_BitStream *bs)
 	case 2:
 		return BD_DecIndexInsert(codec, bs);
 	case 3:
-		return gf_bifs_dec_route(codec, bs, 1);
+		return gf_bifs_dec_route(codec, bs, GF_TRUE);
 	default:
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
@@ -463,7 +689,7 @@ static GF_Err BD_DecIndexDelete(GF_BifsDecoder * codec, GF_BitStream *bs)
 	GF_Node *node;
 	GF_Err e;
 	GF_FieldInfo field;
-	
+
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
@@ -495,7 +721,10 @@ static GF_Err BD_DecIndexDelete(GF_BifsDecoder * codec, GF_BitStream *bs)
 
 	/*special handling in case of a node*/
 	if (SF_type == GF_SG_VRML_SFNODE) {
-		e = gf_node_replace_child(node, (GF_ChildNodeItem**) field.far_ptr, pos, NULL);
+		GF_ChildNodeItem** nlist_ptr = (GF_ChildNodeItem**) field.far_ptr;
+		if (*nlist_ptr) {
+			e = gf_node_replace_child(node, nlist_ptr, pos, NULL);
+		}
 	} else {
 		e = gf_sg_vrml_mf_remove(field.far_ptr, field.fieldType, pos);
 	}
@@ -522,7 +751,7 @@ static GF_Err BD_DecDelete(GF_BifsDecoder * codec, GF_BitStream *bs)
 		if (!n) return GF_OK;
 #endif
 		/*this is a delete of a DEF node, remove ALL INSTANCES*/
-		return gf_node_replace(n, NULL, 0);
+		return gf_node_replace(n, NULL, GF_FALSE);
 	case 2:
 		return BD_DecIndexDelete(codec, bs);
 	case 3:
@@ -542,17 +771,17 @@ static GF_Err BD_DecNodeReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	u32 NodeID;
 	GF_Node *node, *new_node;
 	GF_Err e;
-	
+
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	/*this is delete / new on a DEF node: replace ALL instances*/
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
-	
+
 	/*and just parse a new GF_Node - it is encoded in SFWorldNode table */
 	new_node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
 	if (!new_node && codec->LastError) return codec->LastError;
-	
-	e = gf_node_replace(node, new_node, 0);
+
+	e = gf_node_replace(node, new_node, GF_FALSE);
 	return e;
 }
 
@@ -560,7 +789,7 @@ static GF_Err BD_DecFieldReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
 	GF_Err e;
 	u32 NodeID, ind, field_ind, NumBits;
-	GF_Node *node, *prev_node;
+	GF_Node *node;
 	GF_ChildNodeItem *prev_child;
 	GF_FieldInfo field;
 
@@ -573,31 +802,26 @@ static GF_Err BD_DecFieldReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	if (e) return e;
 
 	e = gf_node_get_field(node, field_ind, &field);
+	if (e) return e;
 
-	prev_node = NULL;
 	prev_child = NULL;
-	/*store prev SF node*/
-	if (field.fieldType == GF_SG_VRML_SFNODE) {
-		prev_node = *((GF_Node **) field.far_ptr);
-	}
 	/*store prev MFNode content*/
-	else if (field.fieldType == GF_SG_VRML_MFNODE) {
+	if (field.fieldType == GF_SG_VRML_MFNODE) {
 		prev_child = * ((GF_ChildNodeItem **) field.far_ptr);
 		* ((GF_ChildNodeItem **) field.far_ptr) = NULL;
 	}
 	/*regular field*/
 	else if (!gf_sg_vrml_is_sf_field(field.fieldType)) {
-		gf_sg_vrml_mf_reset(field.far_ptr, field.fieldType);
+		e = gf_sg_vrml_mf_reset(field.far_ptr, field.fieldType);
+		if (e) return e;
 	}
 
 	/*parse the field*/
-	codec->is_com_dec = 1;
-	e = gf_bifs_dec_field(codec, bs, node, &field);
-	codec->is_com_dec = 0;
+	codec->is_com_dec = GF_TRUE;
+	e = gf_bifs_dec_field(codec, bs, node, &field, GF_FALSE);
+	codec->is_com_dec = GF_FALSE;
 	/*remove prev nodes*/
-	if (field.fieldType == GF_SG_VRML_SFNODE) {
-		if (prev_node) e = gf_node_unregister(prev_node, node);
-	} else if (field.fieldType == GF_SG_VRML_MFNODE) {
+	if (field.fieldType == GF_SG_VRML_MFNODE) {
 		gf_node_unregister_children(node, prev_child);
 	}
 	if (!e) gf_bifs_check_field_change(node, &field);
@@ -624,6 +848,7 @@ static GF_Err BD_DecIndexValueReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	if (e) return e;
 
 	e = gf_node_get_field(node, field_ind, &field);
+	if (e) return e;
 	if (gf_sg_vrml_is_sf_field(field.fieldType)) return GF_NON_COMPLIANT_BITSTREAM;
 
 	type = gf_bs_read_int(bs, 2);
@@ -640,7 +865,7 @@ static GF_Err BD_DecIndexValueReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	default:
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
-	
+
 	/*if MFNode remove the child and parse new node*/
 	if (field.fieldType == GF_SG_VRML_MFNODE) {
 		/*get the new node*/
@@ -669,7 +894,7 @@ static GF_Err BD_DecIndexValueReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 
 		e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, & sffield.far_ptr, pos);
 		if (e) return e;
-		e = gf_bifs_dec_sf_field(codec, bs, node, &sffield);
+		e = gf_bifs_dec_sf_field(codec, bs, node, &sffield, GF_FALSE);
 		if (!e) gf_bifs_check_field_change(node, &field);
 	}
 
@@ -686,14 +911,14 @@ static GF_Err BD_DecRouteReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	GF_Node *OutNode, *InNode;
 
 	RouteID = 1+gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-	
+
 	r = gf_sg_route_find(codec->current_graph, RouteID);
 #ifdef MPEG4_STRICT
 	if (!r) return GF_NON_COMPLIANT_BITSTREAM;
 	ptr = gf_sg_route_get_name(r);
 	gf_sg_route_del(r);
 #else
-	ptr = NULL; 
+	ptr = NULL;
 	if (r) {
 		ptr = gf_sg_route_get_name(r);
 //		gf_sg_route_del(r);
@@ -732,8 +957,14 @@ static GF_Err BD_DecRouteReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 		r->ToNode = InNode;
 		r->ToField.fieldIndex = toID;
 
-		if (!r->FromNode->sgprivate->interact) GF_SAFEALLOC(r->FromNode->sgprivate->interact, struct _node_interactive_ext);
-		if (!r->FromNode->sgprivate->interact->routes) r->FromNode->sgprivate->interact->routes = gf_list_new();
+		if (!r->FromNode->sgprivate->interact) {
+			GF_SAFEALLOC(r->FromNode->sgprivate->interact, struct _node_interactive_ext);
+			if (!r->FromNode->sgprivate->interact) return GF_OUT_OF_MEM;
+		}
+		if (!r->FromNode->sgprivate->interact->routes) {
+			r->FromNode->sgprivate->interact->routes = gf_list_new();
+			if (!r->FromNode->sgprivate->interact->routes) return GF_OUT_OF_MEM;
+		}
 		gf_list_add(r->FromNode->sgprivate->interact->routes, r);
 	} else {
 		r = gf_sg_route_new(codec->current_graph, OutNode, fromID, InNode, toID);
@@ -762,12 +993,13 @@ static GF_Err BD_DecReplace(GF_BifsDecoder * codec, GF_BitStream *bs)
 	return GF_OK;
 }
 
+void command_buffers_del(GF_List *command_buffers);
 
 /*if parent is non-NULL, we are in a proto code parsing, otherwise this is a top-level proto*/
 GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *proto_list)
 {
 	u8 flag, field_type, event_type, useQuant, useAnim, f;
-	u32 i, NbRoutes, ID, numProtos, numFields, count, qpsftype, QP_Type, NumBits, Anim_Type;
+	u32 i, NbRoutes, ID, numProtos, numFields, count, qpsftype, QP_Type, NumBits, nb_cmd_bufs;
 	GF_Node *node;
 	char name[1000];
 	GF_ProtoFieldInterface *proto_field;
@@ -784,6 +1016,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 	ParentProto = codec->pCurrentProto;
 	e = GF_OK;
 
+	nb_cmd_bufs = gf_list_count(codec->command_buffers);
 	numProtos = 0;
 	proto = NULL;
 	flag = gf_bs_read_int(bs, 1);
@@ -795,14 +1028,18 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 		ID = gf_bs_read_int(bs, codec->info->config.ProtoIDBits);
 
 		if (codec->UseName) {
-			gf_bifs_dec_name(bs, name);
+			gf_bifs_dec_name(bs, name, 1000);
 		} else {
-			sprintf(name, "Proto%d", numProtos);
+			sprintf(name, "Proto%d", gf_list_count(codec->current_graph->protos) );
 		}
 		/*create a proto in the current graph*/
-		proto = gf_sg_proto_new(codec->current_graph, ID, name, proto_list ? 1 : 0);
+		proto = gf_sg_proto_new(codec->current_graph, ID, name, proto_list ? GF_TRUE : GF_FALSE);
+		if (!proto) {
+			e = GF_NON_COMPLIANT_BITSTREAM;
+			goto exit;
+		}
 		if (proto_list) gf_list_add(proto_list, proto);
-		
+
 		/*during parsing, this proto is the current active one - all nodes/proto defined/declared
 		below it will belong to its namespace*/
 		codec->current_graph = gf_sg_proto_get_graph(proto);
@@ -813,9 +1050,9 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 		while (flag) {
 			event_type = gf_bs_read_int(bs, 2);
 			field_type = gf_bs_read_int(bs, 6);
-			
+
 			if (codec->UseName) {
-				gf_bifs_dec_name(bs, name);
+				gf_bifs_dec_name(bs, name, 1000);
 			} else {
 				sprintf(name, "_field%d", numFields);
 			}
@@ -831,9 +1068,8 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 			case GF_SG_EVENT_FIELD:
 				/*parse default value except nodes ...*/
 				if (gf_sg_vrml_is_sf_field(field_type)) {
-					e = gf_bifs_dec_sf_field(codec, bs, NULL, &field);
+					e = gf_bifs_dec_sf_field(codec, bs, NULL, &field, GF_FALSE);
 				} else {
-					f = 0;
 					if (codec->info->config.UsePredictiveMFField) {
 						f = gf_bs_read_int(bs, 1);
 						/*predictive encoding of proto field is not possible since QP info is not present yet*/
@@ -873,7 +1109,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 				assert(!flag);
 			}
 			/*reserved*/
-			flag = gf_bs_read_int(bs, 1);
+			gf_bs_read_int(bs, 1);
 
 			/*list or vector*/
 			flag = gf_bs_read_int(bs, 1);
@@ -886,18 +1122,30 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 		}
 		/*get proto code*/
 		else {
+			//scope command buffer solving by proto nodes - cf poc in #2040
+			//this means that any pending command buffer not solved at the end of a proto declaration is
+			//a falty one
+			GF_List *old_cb = codec->command_buffers;
+			codec->command_buffers = gf_list_new();
+			
 			/*parse sub-proto list - subprotos are ALWAYS registered with parent proto graph*/
 			e = gf_bifs_dec_proto_list(codec, bs, NULL);
-			if (e) goto exit;
+			if (e) {
+				command_buffers_del(codec->command_buffers);
+				codec->command_buffers = old_cb;
+				goto exit;
+			}
 
 			flag = 1;
-			
+
 			while (flag) {
 				/*parse all nodes in SFWorldNode table*/
 				node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
 				if (!node) {
 					if (codec->LastError) {
 						e = codec->LastError;
+						command_buffers_del(codec->command_buffers);
+						codec->command_buffers = old_cb;
 						goto exit;
 					} else {
 						flag = gf_bs_read_int(bs, 1);
@@ -905,12 +1153,21 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 					}
 				}
 				e = gf_node_register(node, NULL);
-				if (e) goto exit;
+				if (e) {
+					command_buffers_del(codec->command_buffers);
+					codec->command_buffers = old_cb;
+					goto exit;
+				}
+				//Ivica patch - Flush immediately because of proto instantiation
+				gf_bifs_flush_command_list(codec);
 
 				gf_sg_proto_add_node_code(proto, node);
 				flag = gf_bs_read_int(bs, 1);
 			}
-			
+
+			command_buffers_del(codec->command_buffers);
+			codec->command_buffers = old_cb;
+
 			/*routes*/
 			flag = gf_bs_read_int(bs, 1);
 			if (flag) {
@@ -918,7 +1175,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 				if (flag) {
 					/*list route*/
 					while (flag) {
-						e = gf_bifs_dec_route(codec, bs, 0);
+						e = gf_bifs_dec_route(codec, bs, GF_FALSE);
 						if (e) goto exit;
 						flag = gf_bs_read_int(bs, 1);
 					}
@@ -927,7 +1184,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 					i = gf_bs_read_int(bs, 5);
 					NbRoutes = gf_bs_read_int(bs, i);
 					for (i=0; i<NbRoutes; i++) {
-						e = gf_bifs_dec_route(codec, bs, 0);
+						e = gf_bifs_dec_route(codec, bs, GF_FALSE);
 						if (e) goto exit;
 					}
 				}
@@ -968,16 +1225,16 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 						break;
 					}
 					field.fieldType = qpsftype;
-					
+
 					qp_min_value = gf_sg_vrml_field_pointer_new(qpsftype);
 					field.name = "QPMinValue";
 					field.far_ptr = qp_min_value;
-					gf_bifs_dec_sf_field(codec, bs, NULL, &field);
+					gf_bifs_dec_sf_field(codec, bs, NULL, &field, GF_FALSE);
 
 					qp_max_value = gf_sg_vrml_field_pointer_new(qpsftype);
 					field.name = "QPMaxValue";
 					field.far_ptr = qp_max_value;
-					gf_bifs_dec_sf_field(codec, bs, NULL, &field);
+					gf_bifs_dec_sf_field(codec, bs, NULL, &field, GF_FALSE);
 				}
 
 				/*and store*/
@@ -992,9 +1249,7 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 			if (useAnim && ( (field.eventType == GF_SG_EVENT_IN) || (field.eventType == GF_SG_EVENT_EXPOSED_FIELD) )) {
 				flag = gf_bs_read_int(bs, 1);
 				if (flag) {
-					Anim_Type = gf_bs_read_int(bs, 4);
-				} else {
-					Anim_Type = 0;
+					/*Anim_Type = */gf_bs_read_int(bs, 4);
 				}
 			}
 		}
@@ -1007,7 +1262,19 @@ GF_Err gf_bifs_dec_proto_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List
 
 exit:
 	if (e) {
-		if (proto) gf_sg_proto_del(proto);
+		if (proto) {
+			if (proto_list) gf_list_del_item(proto_list, proto);
+
+			//remove all command buffers inserted while decoding the proto list
+			u32 new_cmd_bufs = gf_list_count(codec->command_buffers);
+			while (nb_cmd_bufs < new_cmd_bufs) {
+				new_cmd_bufs--;
+				CommandBufferItem *cbi = gf_list_pop_back(codec->command_buffers);
+				gf_node_unregister(cbi->node, NULL);
+				gf_free(cbi);
+			}
+			gf_sg_proto_del(proto);
+		}
 		codec->current_graph = rootSG;
 	}
 	/*restore original parent proto at codec level*/
@@ -1032,7 +1299,7 @@ GF_Err gf_bifs_dec_route(GF_BifsDecoder * codec, GF_BitStream *bs, Bool is_inser
 	/*def'ed route*/
 	if (flag) {
 		RouteID = 1 + gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-		if (codec->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name, 1000);
 	}
 	/*origin*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -1043,6 +1310,7 @@ GF_Err gf_bifs_dec_route(GF_BifsDecoder * codec, GF_BitStream *bs, Bool is_inser
 	numBits = gf_get_bit_size(numBits);
 	ind = gf_bs_read_int(bs, numBits);
 	e = gf_bifs_get_field_index(OutNode, ind, GF_SG_FIELD_CODING_OUT, &outField);
+	if (e) return e;
 
 	/*target*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -1076,14 +1344,15 @@ GF_Err BD_DecSceneReplace(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *pro
 	if (!proto_list) gf_sg_reset(codec->current_graph);
 
 	/*reserved*/
-	i = gf_bs_read_int(bs, 6);
+	gf_bs_read_int(bs, 6);
 
-	codec->UseName = gf_bs_read_int(bs, 1);
+	codec->UseName = (Bool)gf_bs_read_int(bs, 1);
 	/*parse PROTOS*/
 	e = gf_bifs_dec_proto_list(codec, bs, proto_list);
 	if (e) goto exit;
 
-	assert(codec->pCurrentProto==NULL);
+	//cannot have replace scene in a proto
+	if (codec->pCurrentProto) return GF_NON_COMPLIANT_BITSTREAM;
 	/*Parse the top node - always of type SFTopNode*/
 	root = gf_bifs_dec_node(codec, bs, NDT_SFTopNode);
 	if (!root && codec->LastError) {
@@ -1100,13 +1369,13 @@ GF_Err BD_DecSceneReplace(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *pro
 	/*Parse the routes*/
 	flag = gf_bs_read_int(bs, 1);
 
-	
+
 	if (flag) {
 		flag = gf_bs_read_int(bs, 1);
 		if (flag) {
 			/*list*/
 			while (flag) {
-				e = gf_bifs_dec_route(codec, bs, 0);
+				e = gf_bifs_dec_route(codec, bs, GF_FALSE);
 				if (e) goto exit;
 				flag = gf_bs_read_int(bs, 1);
 			}
@@ -1115,7 +1384,7 @@ GF_Err BD_DecSceneReplace(GF_BifsDecoder * codec, GF_BitStream *bs, GF_List *pro
 			i = gf_bs_read_int(bs, 5);
 			nbR = gf_bs_read_int(bs, i);
 			for (i=0; i<nbR; i++) {
-				e = gf_bifs_dec_route(codec, bs, 0);
+				e = gf_bifs_dec_route(codec, bs, GF_FALSE);
 				if (e) goto exit;
 			}
 		}
@@ -1153,10 +1422,11 @@ GF_Err gf_bifs_dec_command(GF_BifsDecoder * codec, GF_BitStream *bs)
 	}
 
 	while (gf_list_count(codec->QPs)) {
-		gf_bifs_dec_qp_remove(codec, 1);
+		gf_bifs_dec_qp_remove(codec, GF_TRUE);
 	}
 	gf_bifs_flush_command_list(codec);
 
 	return GF_OK;
 }
 
+#endif /*GPAC_DISABLE_BIFS*/
