@@ -57,7 +57,7 @@ typedef struct
 	Double index;
 	Bool importer;
 	Bool deps, notime, temporal_delim;
-	
+
 	u32 bsdbg;
 
 	//only one input pid declared
@@ -933,7 +933,7 @@ static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 	if (!ctx->opid)
 		return GF_NON_COMPLIANT_BITSTREAM;
-		
+
 	gf_bs_get_content_no_truncate(ctx->state.bs, &ctx->state.frame_obus, &pck_size, &ctx->state.frame_obus_alloc);
 
 	if (!pck_size) {
@@ -1037,7 +1037,12 @@ GF_Err av1dmx_parse_av1(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	if (ctx->timescale && (e==GF_BUFFER_TOO_SMALL))
 		e = GF_OK;
 
-	if (e) return e;
+	if (e) {
+		if (e!=GF_EOS && e!=GF_BUFFER_TOO_SMALL) {
+			av1dmx_parse_flush_sample(filter, ctx);
+		}
+		return e;
+	}
 
 
 	if (!ctx->opid) {
@@ -1392,4 +1397,3 @@ const GF_FilterRegister *av1dmx_register(GF_FilterSession *session)
 	return NULL;
 }
 #endif // GPAC_DISABLE_AV_PARSERS
-
