@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005 
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Management sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -40,44 +41,49 @@ enum
 };
 
 #define REM_TRAIL_MARKS(__str, __sep) while (1) {	\
-		u32 _len = strlen(__str);		\
+		u32 _len = (u32) strlen(__str);		\
 		if (!_len) break;	\
 		_len--;				\
 		if (strchr(__sep, __str[_len])) __str[_len] = 0;	\
 		else break;	\
 	}	\
-
+ 
 
 static GF_Err gf_text_guess_format(char *filename, u32 *fmt)
 {
 	char szLine[2048], szTest[10];
 	u32 val;
-	FILE *test = fopen(filename, "rt");
+	FILE *test = gf_fopen(filename, "rt");
 	if (!test) return GF_URL_ERROR;
 
-	while (fgets(szLine, 2048, test) != NULL) {
+	while (gf_fgets(szLine, 2048, test) != NULL) {
 		REM_TRAIL_MARKS(szLine, "\r\n\t ")
 
 		if (strlen(szLine)) break;
 	}
 	*fmt = GF_TEXT_IMPORT_NONE;
 	if ((szLine[0]=='{') && strstr(szLine, "}{")) *fmt = GF_TEXT_IMPORT_SUB;
-	else if (sscanf(szLine, "%d", &val)==1) {
-		sprintf(szTest, "%d", val);
+	else if (sscanf(szLine, "%u", &val)==1) {
+		sprintf(szTest, "%u", val);
 		if (!strcmp(szTest, szLine)) *fmt = GF_TEXT_IMPORT_SRT;
 	}
 	else if (!strnicmp(szLine, "<?xml ", 6)) {
-		char *ext = strrchr(filename, '.');
+		char *ext = gf_file_ext_start(filename);
 		if (!strnicmp(ext, ".ttxt", 5)) *fmt = GF_TEXT_IMPORT_TTXT;
 		ext = strstr(szLine, "?>");
 		if (ext) ext += 2;
-		if (!ext[0]) fgets(szLine, 2048, test);
+		if (ext && !ext[0]) {
+			if (!gf_fgets(szLine, 2048, test))
+				szLine[0] = '\0';
+		}
 		if (strstr(szLine, "x-quicktime-tx3g")) *fmt = GF_TEXT_IMPORT_TEXML;
 	}
-	fclose(test);
+	gf_fclose(test);
 	return GF_OK;
 }
 
+#ifndef GPAC_DISABLE_VRML
+
 static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxInfo *mux)
 {
 	GF_Err e;
@@ -136,20 +142,20 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 		}
 	}
 
-	srt_in = fopen(mux->file_name, "rt");
+	srt_in = gf_fopen(mux->file_name, "rt");
 	if (!srt_in) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[srt->bifs] cannot open input file %s\n", mux->file_name));
 		return GF_URL_ERROR;
 	}
 
-	srt = gf_sm_stream_new(ctx, src->ESID, GF_STREAM_SCENE, 1);
+	srt = gf_sm_stream_new(ctx, src->ESID, GF_STREAM_SCENE, GF_CODECID_BIFS);
 	if (!srt) return GF_OUT_OF_MEM;
 
 	if (!src->slConfig) src->slConfig = (GF_SLConfig *) gf_odf_desc_new(GF_ODF_SLC_TAG);
 	src->slConfig->timestampResolution = 1000;
 	if (!src->decoderConfig) src->decoderConfig = (GF_DecoderConfig *) gf_odf_desc_new(GF_ODF_DCD_TAG);
 	src->decoderConfig->streamType = GF_STREAM_SCENE;
-	src->decoderConfig->objectTypeIndication = 1;
+	src->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 
 	e = GF_OK;
 	state = end = 0;
@@ -160,63 +166,63 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 	inf = NULL;
 
 	while (1) {
-		char *sOK = fgets(szLine, 2048, srt_in);
+		char *sOK = gf_fgets(szLine, 2048, srt_in);
 
 		if (sOK) REM_TRAIL_MARKS(szLine, "\r\n\t ")
 
-		if (!sOK || !strlen(szLine)) {
-			state = 0;
-			if (au) {
-				/*if italic or underscore do it*/
-				if (font && (italic || underlined || bold)) {
-					com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-					com->node = font;
-					gf_node_register(font, NULL);
-					inf = gf_sg_command_field_new(com);
-					inf->fieldIndex = style.fieldIndex;
-					inf->fieldType = style.fieldType;
-					inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
-					sfstr = (SFString *)inf->field_ptr;
-					if (bold && italic && underlined) sfstr->buffer = strdup("BOLDITALIC UNDERLINED");
-					else if (italic && underlined) sfstr->buffer = strdup("ITALIC UNDERLINED");
-					else if (bold && underlined) sfstr->buffer = strdup("BOLD UNDERLINED");
-					else if (underlined) sfstr->buffer = strdup("UNDERLINED");
-					else if (bold && italic) sfstr->buffer = strdup("BOLDITALIC");
-					else if (bold) sfstr->buffer = strdup("BOLD");
-					else sfstr->buffer = strdup("ITALIC");
-					gf_list_add(au->commands, com);
-				}
+			if (!sOK || !strlen(szLine)) {
+				state = 0;
+				if (au) {
+					/*if italic or underscore do it*/
+					if (font && (italic || underlined || bold)) {
+						com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
+						com->node = font;
+						gf_node_register(font, NULL);
+						inf = gf_sg_command_field_new(com);
+						inf->fieldIndex = style.fieldIndex;
+						inf->fieldType = style.fieldType;
+						inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
+						sfstr = (SFString *)inf->field_ptr;
+						if (bold && italic && underlined) sfstr->buffer = gf_strdup("BOLDITALIC UNDERLINED");
+						else if (italic && underlined) sfstr->buffer = gf_strdup("ITALIC UNDERLINED");
+						else if (bold && underlined) sfstr->buffer = gf_strdup("BOLD UNDERLINED");
+						else if (underlined) sfstr->buffer = gf_strdup("UNDERLINED");
+						else if (bold && italic) sfstr->buffer = gf_strdup("BOLDITALIC");
+						else if (bold) sfstr->buffer = gf_strdup("BOLD");
+						else sfstr->buffer = gf_strdup("ITALIC");
+						gf_list_add(au->commands, com);
+					}
 
-				au = gf_sm_stream_au_new(srt, end, 0, 1);
-				com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-				com->node = text;
-				gf_node_register(text, NULL);
-				inf = gf_sg_command_field_new(com);
-				inf->fieldIndex = string.fieldIndex;
-				inf->fieldType = string.fieldType;
-				inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
-				gf_list_add(au->commands, com);
-				/*reset font styles so that all AUs are true random access*/
-				if (font) {
+					au = gf_sm_stream_au_new(srt, end, 0, 1);
 					com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-					com->node = font;
-					gf_node_register(font, NULL);
+					com->node = text;
+					gf_node_register(text, NULL);
 					inf = gf_sg_command_field_new(com);
-					inf->fieldIndex = style.fieldIndex;
-					inf->fieldType = style.fieldType;
-					inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
+					inf->fieldIndex = string.fieldIndex;
+					inf->fieldType = string.fieldType;
+					inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
 					gf_list_add(au->commands, com);
+					/*reset font styles so that all AUs are true random access*/
+					if (font) {
+						com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
+						com->node = font;
+						gf_node_register(font, NULL);
+						inf = gf_sg_command_field_new(com);
+						inf->fieldIndex = style.fieldIndex;
+						inf->fieldType = style.fieldType;
+						inf->field_ptr = gf_sg_vrml_field_pointer_new(style.fieldType);
+						gf_list_add(au->commands, com);
+					}
+					au = NULL;
 				}
-				au = NULL;
+				inf = NULL;
+				if (!sOK) break;
+				continue;
 			}
-			inf = NULL;
-			if (!sOK) break;
-			continue;
-		}
 
 		switch (state) {
 		case 0:
-			if (sscanf(szLine, "%d", &line) != 1) {
+			if (sscanf(szLine, "%u", &line) != 1) {
 				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[srt->bifs] bad frame format (src: %s)\n", szLine));
 				e = GF_CORRUPTED_DATA;
 				goto exit;
@@ -230,8 +236,8 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			state = 1;
 			break;
 		case 1:
-			if (sscanf(szLine, "%d:%d:%d,%d --> %d:%d:%d,%d", &sh, &sm, &ss, &sms, &eh, &em, &es, &ems) != 8) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[srt->bifs] bad frame %d (src: %s)\n", curLine, szLine));
+			if (sscanf(szLine, "%u:%u:%u,%u --> %u:%u:%u,%u", &sh, &sm, &ss, &sms, &eh, &em, &es, &ems) != 8) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[srt->bifs] bad frame %u (src: %s)\n", curLine, szLine));
 				e = GF_CORRUPTED_DATA;
 				goto exit;
 			}
@@ -256,7 +262,7 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 
 			au = gf_sm_stream_au_new(srt, start, 0, 1);
 			com = NULL;
-			state = 2;			
+			state = 2;
 			italic = underlined = bold = 0;
 			break;
 
@@ -305,7 +311,7 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 						szText[len] = 0xc0 | ( (ptr[i] >> 6) & 0x3 );
 						len++;
 						ptr[i] &= 0xbf;
-					} 
+					}
 					/*we only handle UTF8 chars on 2 bytes (eg first byte is 0b110xxxxx)*/
 					else if ((ptr[i] & 0xe0) == 0xc0) {
 						szText[len] = ptr[i];
@@ -317,16 +323,19 @@ static GF_Err gf_text_import_srt_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 				len++;
 			}
 			szText[len] = 0;
-			sfstr->buffer = strdup(szText);
+			sfstr->buffer = gf_strdup(szText);
 			break;
 		}
 	}
 
 exit:
-	if (e) gf_sm_stream_del(ctx, srt);
-	fclose(srt_in);
+	gf_fclose(srt_in);
 	return e;
 }
+#endif /*GPAC_DISABLE_VRML*/
+
+
+#ifndef GPAC_DISABLE_VRML
 
 static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxInfo *mux)
 {
@@ -341,7 +350,6 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 	SFString *sfstr;
 	GF_CommandField *inf;
 	Bool first_samp;
-	Double fps;
 	char szLine[2048], szTime[30], szText[2048];
 	GF_StreamContext *sc = NULL;
 
@@ -386,39 +394,36 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 		}
 	}
 
-	sub_in = fopen(mux->file_name, "rt");
+	sub_in = gf_fopen(mux->file_name, "rt");
 	if (!sub_in) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[sub->bifs] cannot open input file %s\n", mux->file_name));
 		return GF_URL_ERROR;
 	}
 
-	srt = gf_sm_stream_new(ctx, src->ESID, GF_STREAM_SCENE, 1);
+	srt = gf_sm_stream_new(ctx, src->ESID, GF_STREAM_SCENE, GF_CODECID_BIFS);
 	if (!srt) return GF_OUT_OF_MEM;
 
 	if (!src->slConfig) src->slConfig = (GF_SLConfig *) gf_odf_desc_new(GF_ODF_SLC_TAG);
 	src->slConfig->timestampResolution = 1000;
 	if (!src->decoderConfig) src->decoderConfig = (GF_DecoderConfig *) gf_odf_desc_new(GF_ODF_DCD_TAG);
 	src->decoderConfig->streamType = GF_STREAM_SCENE;
-	src->decoderConfig->objectTypeIndication = 1;
+	src->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 
 	e = GF_OK;
-	start = end = 0;
+	end = 0;
 	au = NULL;
 	com = NULL;
 	inf = NULL;
 
-	fps = 25.0;
-	if (mux->frame_rate) fps = mux->frame_rate;
-
 	line = 0;
 	first_samp = 1;
 	while (1) {
-		char *sOK = fgets(szLine, 2048, sub_in);
+		char *sOK = gf_fgets(szLine, 2048, sub_in);
 		if (!sOK) break;
 		REM_TRAIL_MARKS(szLine, "\r\n\t ")
 
 		line++;
-		len = strlen(szLine); 
+		len = (u32) strlen(szLine);
 		if (!len) continue;
 
 		i=0;
@@ -427,7 +432,10 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			e = GF_NON_COMPLIANT_BITSTREAM;
 			break;
 		}
-		while (szLine[i+1] && szLine[i+1]!='}') { szTime[i] = szLine[i+1]; i++; }
+		while (szLine[i+1] && szLine[i+1]!='}') {
+			szTime[i] = szLine[i+1];
+			i++;
+		}
 		szTime[i] = 0;
 		start = atoi(szTime);
 		if (start<end) {
@@ -441,7 +449,10 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 			e = GF_NON_COMPLIANT_BITSTREAM;
 			break;
 		}
-		while (szLine[i+1+j] && szLine[i+1+j]!='}') { szTime[i] = szLine[i+1+j]; i++; }
+		while (szLine[i+1+j] && szLine[i+1+j]!='}') {
+			szTime[i] = szLine[i+1+j];
+			i++;
+		}
 		szTime[i] = 0;
 		end = atoi(szTime);
 		j+=i+2;
@@ -488,27 +499,30 @@ static GF_Err gf_text_import_sub_bifs(GF_SceneManager *ctx, GF_ESD *src, GF_MuxI
 		}
 		szText[i-j] = 0;
 
-		com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
-		com->node = text;
-		gf_node_register(text, NULL);
-		inf = gf_sg_command_field_new(com);
-		inf->fieldIndex = string.fieldIndex;
-		inf->fieldType = string.fieldType;
-		inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
-		gf_list_add(au->commands, com);
+		if (au) {
+			com = gf_sg_command_new(ctx->scene_graph, GF_SG_FIELD_REPLACE);
+			com->node = text;
+			gf_node_register(text, NULL);
+			inf = gf_sg_command_field_new(com);
+			inf->fieldIndex = string.fieldIndex;
+			inf->fieldType = string.fieldType;
+			inf->field_ptr = gf_sg_vrml_field_pointer_new(string.fieldType);
+			gf_list_add(au->commands, com);
 
-		gf_sg_vrml_mf_append(inf->field_ptr, GF_SG_VRML_MFSTRING, (void **) &sfstr);
-		sfstr->buffer = strdup(szText);
+			gf_sg_vrml_mf_append(inf->field_ptr, GF_SG_VRML_MFSTRING, (void **) &sfstr);
+			sfstr->buffer = gf_strdup(szText);
+		}
 	}
-
-	if (e) gf_sm_stream_del(ctx, srt);
-	fclose(sub_in);
+	gf_fclose(sub_in);
 	return e;
 }
+#endif
+
 
 GF_EXPORT
 GF_Err gf_sm_import_bifs_subtitle(GF_SceneManager *ctx, GF_ESD *src, GF_MuxInfo *mux)
 {
+#ifndef GPAC_DISABLE_VRML
 	GF_Err e;
 	u32 fmt;
 	e = gf_text_guess_format(mux->file_name, &fmt);
@@ -517,5 +531,8 @@ GF_Err gf_sm_import_bifs_subtitle(GF_SceneManager *ctx, GF_ESD *src, GF_MuxInfo
 
 	if (fmt==1) return gf_text_import_srt_bifs(ctx, src, mux);
 	else return gf_text_import_sub_bifs(ctx, src, mux);
+#else
+	return GF_NOT_SUPPORTED;
+#endif
 }
 
