@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Graph sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -27,6 +28,9 @@
 /*MPEG4 tags (for internal nodes)*/
 #include <gpac/nodes_mpeg4.h>
 
+
+#ifndef GPAC_DISABLE_VRML
+
 static Fixed Interpolate(Fixed keyValue1, Fixed keyValue2, Fixed fraction)
 {
 	return gf_mulfix(keyValue2 - keyValue1, fraction) + keyValue1;
@@ -94,9 +98,9 @@ static Fixed quadratic_knots[] = {0,0,0,FIX_ONE,FIX_ONE,FIX_ONE};
 
 static void anurbs_reset(anim_nurbs *nurbs)
 {
-	if (nurbs->n) free(nurbs->n);
-	if (nurbs->left) free(nurbs->left);
-	if (nurbs->right) free(nurbs->right);
+	if (nurbs->n) gf_free(nurbs->n);
+	if (nurbs->left) gf_free(nurbs->left);
+	if (nurbs->right) gf_free(nurbs->right);
 	nurbs->n = nurbs->left = nurbs->right = NULL;
 }
 
@@ -124,8 +128,8 @@ static void anurbs_init(anim_nurbs *nurbs, u32 type, u32 nCtrl, u32 nKnots, Fixe
 		break;
 	}
 	nurbs->p = nurbs->nknots - nurbs->npoints - 1;
-	if ((nurbs->p<=0) || (nurbs->p >= nurbs->nknots -1) 
-		|| ((nurbs->nweights>0) && (nurbs->npoints != nurbs->nweights)) ) {
+	if ((nurbs->p<=0) || (nurbs->p >= nurbs->nknots -1)
+	        || ((nurbs->nweights>0) && (nurbs->npoints != nurbs->nweights)) ) {
 		nurbs->valid = 0;
 	} else {
 		nurbs->valid = 1;
@@ -137,9 +141,9 @@ static void anurbs_basis(anim_nurbs *nurbs, s32 span, Fixed t)
 	u32 i, j;
 	Fixed saved, temp;
 	if (!nurbs->n) {
-		nurbs->n = (Fixed*)malloc(sizeof(Fixed) * (nurbs->p+1));
-		nurbs->left = (Fixed*)malloc(sizeof(Fixed) * (nurbs->p+1));
-		nurbs->right = (Fixed*)malloc(sizeof(Fixed) * (nurbs->p+1));
+		nurbs->n = (Fixed*)gf_malloc(sizeof(Fixed) * (nurbs->p+1));
+		nurbs->left = (Fixed*)gf_malloc(sizeof(Fixed) * (nurbs->p+1));
+		nurbs->right = (Fixed*)gf_malloc(sizeof(Fixed) * (nurbs->p+1));
 	}
 	nurbs->n[0] = FIX_ONE;
 
@@ -147,7 +151,7 @@ static void anurbs_basis(anim_nurbs *nurbs, s32 span, Fixed t)
 		nurbs->left[i] = t - nurbs->knots[span+1-i];
 		nurbs->right[i] = nurbs->knots[span+i]-t;
 		saved = 0;
-		
+
 		for(j=0; j<i; j++) {
 			temp = gf_divfix(nurbs->n[j], nurbs->right[j+1] + nurbs->left[i-j]);
 			nurbs->n[j] = saved + gf_mulfix(nurbs->right[j+1], temp);
@@ -168,18 +172,18 @@ static s32 anurbs_find_span(anim_nurbs *nurbs, Fixed u)
 	span--;
 	return span;
 #else
-	s32 low, high, mid;	
+	s32 low, high, mid;
 	if (u == nurbs->knots[nurbs->npoints]) return nurbs->npoints - 1;
-	low = nurbs->p; 
-	high = nurbs->npoints; 
+	low = nurbs->p;
+	high = nurbs->npoints;
 	mid = (low + high)/2;
-	
+
 	while (u < nurbs->knots[mid] || u >= nurbs->knots[mid+1]) {
-		if (u < nurbs->knots[mid]) high = mid; 
+		if (u < nurbs->knots[mid]) high = mid;
 		else low = mid;
 		mid = (low + high)/2;
-	}	
-	return (mid); 
+	}
+	return (mid);
 
 #endif
 }
@@ -192,7 +196,7 @@ static SFVec3f anurbs_get_vec3f(anim_nurbs *nurbs, s32 span, SFVec3f *pts)
 	tmp.x = tmp.y = tmp.z = 0;
 	res = tmp;
 	w=0;
-	for(i=0; i<=nurbs->p;i++) {
+	for(i=0; i<=nurbs->p; i++) {
 		tmp = pts[span - nurbs->p + i];
 		if (nurbs->nweights>0) {
 			wi = nurbs->weights[span - nurbs->p + i];
@@ -220,7 +224,7 @@ static SFVec2f anurbs_get_vec2f(anim_nurbs *nurbs, s32 span, SFVec2f *pts)
 	tmp.x = tmp.y = 0;
 	res = tmp;
 	w=0;
-	for(i=0; i<=nurbs->p;i++) {
+	for(i=0; i<=nurbs->p; i++) {
 		tmp = pts[span - nurbs->p + i];
 		if (nurbs->nweights>0) {
 			wi = nurbs->weights[span - nurbs->p + i];
@@ -243,13 +247,13 @@ static SFVec2f anurbs_get_vec2f(anim_nurbs *nurbs, s32 span, SFVec2f *pts)
 
 static Fixed anurbs_get_float(anim_nurbs *nurbs, s32 span, Fixed *vals)
 {
-	Fixed res, tmp;
+	Fixed res;
 	Fixed w, wi;
 	u32 i;
-	res = tmp = 0;
+	res = 0;
 	w=0;
-	for(i=0; i<=nurbs->p;i++) {
-		tmp = vals[span - nurbs->p + i];
+	for(i=0; i<=nurbs->p; i++) {
+		Fixed tmp = vals[span - nurbs->p + i];
 		if (nurbs->nweights>0) {
 			wi = nurbs->weights[span - nurbs->p + i];
 			tmp = gf_mulfix(tmp, wi);
@@ -278,7 +282,7 @@ static void Anim_Destroy(GF_Node *node, void *rs, Bool is_destroy)
 	if (is_destroy) {
 		AnimatorStack *stack = (AnimatorStack *)gf_node_get_private(node);
 		anurbs_reset(&stack->anurbs);
-		free(stack);
+		gf_free(stack);
 	}
 }
 
@@ -336,10 +340,10 @@ static void PA_Update(M_PositionAnimator *pa, AnimatorStack *stack)
 			d.z = pa->keyValue.vals[i+1].z - pa->keyValue.vals[i].z;
 			stack->length += gf_vec_len(d);
 		}
-	} 
+	}
 	Animator_Update(stack, pa->keyValueType, pa->keyValue.count, &pa->keySpline, pa->weight.count, pa->weight.vals);
 }
-static void PA_SetFraction(GF_Node *node)
+static void PA_SetFraction(GF_Node *node, GF_Route *route)
 {
 	Fixed frac;
 	u32 nbKeys, nbVals, i;
@@ -356,7 +360,6 @@ static void PA_SetFraction(GF_Node *node)
 	nbKeys = pa->key.count;
 	nbVals = pa->keyValue.count;
 
-	i=0;
 	switch (pa->keyValueType) {
 	/*linear interpolate*/
 	case ANIM_LINE:
@@ -364,8 +367,14 @@ static void PA_SetFraction(GF_Node *node)
 		switch (stack->anim_type) {
 		case ANIM_DEFAULT:
 			if (nbKeys != nbVals) return;
-			if (frac<pa->key.vals[0]) { i=0; frac = 0; }
-			else if (frac>pa->key.vals[nbKeys-1]) { i=nbVals-2; frac = FIX_ONE; }
+			if (frac<pa->key.vals[0]) {
+				i=0;
+				frac = 0;
+			}
+			else if (frac>pa->key.vals[nbKeys-1]) {
+				i=nbVals-2;
+				frac = FIX_ONE;
+			}
 			else {
 				for (i=0; i<nbKeys-1; i++) {
 					if ((frac>=pa->key.vals[i]) && (frac<pa->key.vals[i+1])) break;
@@ -432,7 +441,7 @@ static void PA_SetFraction(GF_Node *node)
 		/*nothing to do for this one here*/
 		case ANIM_DEFAULT:
 		/*not supported - use frac as is*/
-		case ANIM_PACED: 
+		case ANIM_PACED:
 		default:
 			break;
 		}
@@ -442,7 +451,7 @@ static void PA_SetFraction(GF_Node *node)
 		pa->value_changed = anurbs_get_vec3f(&stack->anurbs, i, pa->keyValue.vals);
 		break;
 	/*not supported*/
-	case ANIM_SPLINE: 
+	case ANIM_SPLINE:
 	default:
 		return;
 	}
@@ -450,7 +459,7 @@ static void PA_SetFraction(GF_Node *node)
 	pa->value_changed.x += pa->offset.x;
 	pa->value_changed.y += pa->offset.y;
 	pa->value_changed.z += pa->offset.z;
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 12/*"value_changed"*/);
 }
 
 void PA_Modified(GF_Node *node, GF_FieldInfo *field)
@@ -459,21 +468,24 @@ void PA_Modified(GF_Node *node, GF_FieldInfo *field)
 	M_PositionAnimator *pa = (M_PositionAnimator *)node;
 
 	if ( /*all fields impacting cached path len / nurbs*/
-		(field->far_ptr == &pa->keyValue) 
-		|| (field->far_ptr == &pa->keyValueType) 
-		|| (field->far_ptr == &pa->key) 
-		|| (field->far_ptr == &pa->keyType) 
-		|| (field->far_ptr == &pa->keySpline) 
-		|| (field->far_ptr == &pa->weight) 
-		|| (field->far_ptr == &pa->key) 
-		
-		) stack->is_dirty = 1;
+	    (field->far_ptr == &pa->keyValue)
+	    || (field->far_ptr == &pa->keyValueType)
+	    || (field->far_ptr == &pa->key)
+	    || (field->far_ptr == &pa->keyType)
+	    || (field->far_ptr == &pa->keySpline)
+	    || (field->far_ptr == &pa->weight)
+	)
+		stack->is_dirty = 1;
 }
 void PA_Init(GF_Node *n)
 {
 	M_PositionAnimator *sa = (M_PositionAnimator*)n;
 	AnimatorStack *stack;
 	GF_SAFEALLOC(stack, AnimatorStack);
+	if (!stack) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("[VRML] Failed to allocate position animator stack\n"));
+		return;
+	}
 	stack->is_dirty = 1;
 	gf_node_set_private(n, stack);
 	gf_node_set_callback_function(n, Anim_Destroy);
@@ -496,10 +508,10 @@ static void PA2D_Update(M_PositionAnimator2D *pa, AnimatorStack *stack)
 			dy = pa->keyValue.vals[i+1].y - pa->keyValue.vals[i].y;
 			stack->length += gf_sqrt(gf_mulfix(dx, dx) + gf_mulfix(dy, dy));
 		}
-	} 
+	}
 	Animator_Update(stack, pa->keyValueType, pa->keyValue.count, &pa->keySpline, pa->weight.count, pa->weight.vals);
 }
-static void PA2D_SetFraction(GF_Node *node)
+static void PA2D_SetFraction(GF_Node *node, GF_Route *route)
 {
 	Fixed frac;
 	u32 nbKeys, nbVals, i;
@@ -516,7 +528,6 @@ static void PA2D_SetFraction(GF_Node *node)
 	nbKeys = pa->key.count;
 	nbVals = pa->keyValue.count;
 
-	i=0;
 	switch (pa->keyValueType) {
 	/*linear interpolate*/
 	case ANIM_LINE:
@@ -524,8 +535,14 @@ static void PA2D_SetFraction(GF_Node *node)
 		switch (stack->anim_type) {
 		case ANIM_DEFAULT:
 			if (nbKeys != nbVals) return;
-			if (frac<=pa->key.vals[0]) { i=0; frac = 0; }
-			else if (frac>=pa->key.vals[nbKeys-1]) { i=nbVals-2; frac=FIX_ONE; }
+			if (frac<=pa->key.vals[0]) {
+				i=0;
+				frac = 0;
+			}
+			else if (frac>=pa->key.vals[nbKeys-1]) {
+				i=nbVals-2;
+				frac=FIX_ONE;
+			}
 			else {
 				for (i=0; i<nbKeys-1; i++) {
 					if ((frac>=pa->key.vals[i]) && (frac<pa->key.vals[i+1])) break;
@@ -590,7 +607,7 @@ static void PA2D_SetFraction(GF_Node *node)
 		/*nothing to do for this one here*/
 		case ANIM_DEFAULT:
 		/*not supported - use frac as is*/
-		case ANIM_PACED: 
+		case ANIM_PACED:
 		default:
 			break;
 		}
@@ -600,14 +617,14 @@ static void PA2D_SetFraction(GF_Node *node)
 		pa->value_changed = anurbs_get_vec2f(&stack->anurbs, i, pa->keyValue.vals);
 		break;
 	/*not supported*/
-	case ANIM_SPLINE: 
+	case ANIM_SPLINE:
 	default:
 		return;
 	}
 
 	pa->value_changed.x += pa->offset.x;
 	pa->value_changed.y += pa->offset.y;
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 12/*"value_changed"*/);
 }
 
 void PA2D_Modified(GF_Node *node, GF_FieldInfo *field)
@@ -616,21 +633,24 @@ void PA2D_Modified(GF_Node *node, GF_FieldInfo *field)
 	M_PositionAnimator2D *pa = (M_PositionAnimator2D *)node;
 
 	if ( /*all fields impacting cached path len / nurbs*/
-		(field->far_ptr == &pa->keyValue) 
-		|| (field->far_ptr == &pa->keyValueType) 
-		|| (field->far_ptr == &pa->key) 
-		|| (field->far_ptr == &pa->keyType) 
-		|| (field->far_ptr == &pa->keySpline) 
-		|| (field->far_ptr == &pa->weight) 
-		|| (field->far_ptr == &pa->key) 
-		
-		) stack->is_dirty = 1;
+	    (field->far_ptr == &pa->keyValue)
+	    || (field->far_ptr == &pa->keyValueType)
+	    || (field->far_ptr == &pa->key)
+	    || (field->far_ptr == &pa->keyType)
+	    || (field->far_ptr == &pa->keySpline)
+	    || (field->far_ptr == &pa->weight)
+	)
+		stack->is_dirty = 1;
 }
 void PA2D_Init(GF_Node *n)
 {
 	M_PositionAnimator2D *sa = (M_PositionAnimator2D *)n;
 	AnimatorStack *stack;
 	GF_SAFEALLOC(stack, AnimatorStack);
+	if (!stack) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("[VRML] Failed to allocate position animator 2D stack\n"));
+		return;
+	}
 	stack->is_dirty = 1;
 	gf_node_set_private(n, stack);
 	gf_node_set_callback_function(n, Anim_Destroy);
@@ -652,11 +672,11 @@ static void SA_Update(M_ScalarAnimator *sa, AnimatorStack *stack)
 			len = sa->keyValue.vals[i+1] - sa->keyValue.vals[i];
 			stack->length += ABS(len);
 		}
-	} 
+	}
 	Animator_Update(stack, sa->keyValueType, sa->keyValue.count, &sa->keySpline, sa->weight.count, sa->weight.vals);
 }
 
-void SA_SetFraction(GF_Node *node)
+void SA_SetFraction(GF_Node *node, GF_Route *route)
 {
 	Fixed frac;
 	u32 nbKeys, nbVals, i;
@@ -680,8 +700,14 @@ void SA_SetFraction(GF_Node *node)
 		switch (stack->anim_type) {
 		case ANIM_DEFAULT:
 			if (nbKeys != nbVals) return;
-			if (frac<sa->key.vals[0]) { i=0; frac = 0; }
-			else if (frac>sa->key.vals[nbKeys-1]) { i=nbVals-2; frac=FIX_ONE; }
+			if (frac<sa->key.vals[0]) {
+				i=0;
+				frac = 0;
+			}
+			else if (frac>sa->key.vals[nbKeys-1]) {
+				i=nbVals-2;
+				frac=FIX_ONE;
+			}
 			else {
 				for (i=0; i<nbKeys-1; i++) {
 					if ((frac>=sa->key.vals[i]) && (frac<sa->key.vals[i+1])) break;
@@ -742,7 +768,7 @@ void SA_SetFraction(GF_Node *node)
 		/*nothing to do for this one here*/
 		case ANIM_DEFAULT:
 		/*not supported - use frac as is*/
-		case ANIM_PACED: 
+		case ANIM_PACED:
 		default:
 			break;
 		}
@@ -752,13 +778,13 @@ void SA_SetFraction(GF_Node *node)
 		sa->value_changed = anurbs_get_float(&stack->anurbs, i, sa->keyValue.vals);
 		break;
 	/*not supported*/
-	case ANIM_SPLINE: 
+	case ANIM_SPLINE:
 	default:
 		return;
 	}
 
 	sa->value_changed += sa->offset;
-	gf_node_event_out_str(node, "value_changed");
+	gf_node_event_out(node, 10/*"value_changed"*/);
 }
 
 void SA_Modified(GF_Node *node, GF_FieldInfo *field)
@@ -767,15 +793,14 @@ void SA_Modified(GF_Node *node, GF_FieldInfo *field)
 	M_ScalarAnimator *sa = (M_ScalarAnimator *)node;
 
 	if ( /*all fields impacting cached path len / nurbs*/
-		(field->far_ptr == &sa->keyValue) 
-		|| (field->far_ptr == &sa->keyValueType) 
-		|| (field->far_ptr == &sa->key) 
-		|| (field->far_ptr == &sa->keyType) 
-		|| (field->far_ptr == &sa->keySpline) 
-		|| (field->far_ptr == &sa->weight) 
-		|| (field->far_ptr == &sa->key) 
-		
-		) stack->is_dirty = 1;
+	    (field->far_ptr == &sa->keyValue)
+	    || (field->far_ptr == &sa->keyValueType)
+	    || (field->far_ptr == &sa->key)
+	    || (field->far_ptr == &sa->keyType)
+	    || (field->far_ptr == &sa->keySpline)
+	    || (field->far_ptr == &sa->weight)
+	)
+		stack->is_dirty = 1;
 }
 
 void SA_Init(GF_Node *n)
@@ -783,6 +808,10 @@ void SA_Init(GF_Node *n)
 	M_ScalarAnimator *sa = (M_ScalarAnimator *)n;
 	AnimatorStack *stack;
 	GF_SAFEALLOC(stack, AnimatorStack);
+	if (!stack) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_SCENE, ("[VRML] Failed to allocate scalar animator stack\n"));
+		return;
+	}
 	stack->is_dirty = 1;
 	gf_node_set_private(n, stack);
 	gf_node_set_callback_function(n, Anim_Destroy);
@@ -790,3 +819,4 @@ void SA_Init(GF_Node *n)
 }
 
 
+#endif	/*GPAC_DISABLE_VRML*/
