@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2005-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / common tools sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -35,15 +36,15 @@
 #include <sys/times.h>
 #include <sys/resource.h>
 /*symbian stdlib*/
-#include <e32std.h> 
+#include <e32std.h>
 /*symbian core (for scheduler & trap cleanup)*/
-#include <e32base.h> 
+#include <e32base.h>
 /*hardware abstraction layer*/
 #include <hal.h>
- 
+
 /*gpac module internals*/
 #include "module_wrap.h"
-#include <gpac/thread.h> 
+#include <gpac/thread.h>
 
 
 //  Exported Functions (DLL entry point)
@@ -78,21 +79,21 @@ void gf_sleep(u32 ms)
 	TTimeIntervalMicroSeconds32 inter;
 	inter = (TInt) (1000*ms);
 #ifdef __SERIES60_3X__
-	User::AfterHighRes(inter); 
+	User::AfterHighRes(inter);
 #else
-	User::After(inter); 
+	User::After(inter);
 #endif
 
 #if 0
 	TInt error;
-    CActiveScheduler::RunIfReady(error, CActive::EPriorityIdle);
+	CActiveScheduler::RunIfReady(error, CActive::EPriorityIdle);
 
 	RTimer timer;
 	TRequestStatus timerStatus;
 	timer.CreateLocal();
-	
+
 	timer.After(timerStatus,ms*1000);
-	User::WaitForRequest(timerStatus); 
+	User::WaitForRequest(timerStatus);
 #endif
 }
 
@@ -124,7 +125,7 @@ u32 gf_rand()
 GF_EXPORT
 FILE *gf_temp_file_new()
 {
-	return tmpfile(); 
+	return tmpfile();
 }
 #endif
 
@@ -234,19 +235,19 @@ next:
 
 
 GF_EXPORT
-u64 gf_f64_tell(FILE *fp)
+u64 gf_ftell(FILE *fp)
 {
 	return (u64) ftell(fp);
 }
 
 GF_EXPORT
-u64 gf_f64_seek(FILE *fp, s64 offset, s32 whence)
+u64 gf_fseek(FILE *fp, s64 offset, s32 whence)
 {
 	return fseek(fp, (s32) offset, whence);
 }
 
 GF_EXPORT
-FILE *gf_f64_open(const char *file_name, const char *mode)
+FILE *gf_fopen(const char *file_name, const char *mode)
 {
 	return fopen(file_name, mode);
 }
@@ -277,16 +278,16 @@ struct __tag_thread
 GF_EXPORT
 GF_Thread *gf_th_new(const char *name)
 {
-	GF_Thread *tmp = (GF_Thread *) malloc(sizeof(GF_Thread));
+	GF_Thread *tmp = (GF_Thread *) gf_malloc(sizeof(GF_Thread));
 	memset((void *)tmp, 0, sizeof(GF_Thread));
 	tmp->status = GF_THREAD_STATUS_STOP;
 #ifndef GPAC_DISABLE_LOG
 	if (name) {
-		tmp->log_name = strdup(name);
+		tmp->log_name = gf_strdup(name);
 	} else {
 		char szN[20];
 		sprintf(szN, "0x%08x", (u32) tmp);
-		tmp->log_name = strdup(szN);
+		tmp->log_name = gf_strdup(szN);
 	}
 #endif
 
@@ -306,7 +307,7 @@ static void *RunThread(void *ptr)
 		t->status = GF_THREAD_STATUS_DEAD;
 		gf_sema_notify(t->_signal, 1);
 		goto exit;
-	} 
+	}
 #ifndef GPAC_DISABLE_LOG
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Thread %s] Installing ActiveScheduler\n", t->log_name));
 #endif
@@ -369,7 +370,7 @@ GF_Err gf_th_run(GF_Thread *t, u32 (*Run)(void *param), void *param)
 
 	threadName.Format(_L("GTH%d"), (u32) t);
 	t->threadH = new RThread();
-	if ( t->threadH->Create(threadName, (TThreadFunction)RunThread, KDefaultStackSize, KThreadMinHeapSize, KThreadMaxHeapSize, (void *)t, EOwnerProcess) != KErrNone){
+	if ( t->threadH->Create(threadName, (TThreadFunction)RunThread, KDefaultStackSize, KThreadMinHeapSize, KThreadMaxHeapSize, (void *)t, EOwnerProcess) != KErrNone) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[Core] Unable to create thread\n"));
 		t->status = GF_THREAD_STATUS_DEAD;
 		return GF_IO_ERR;
@@ -377,7 +378,7 @@ GF_Err gf_th_run(GF_Thread *t, u32 (*Run)(void *param), void *param)
 	t->threadH->Resume();
 
 	/*wait for the child function to call us - do NOT return before, otherwise the thread status would
-	be unknown*/ 	
+	be unknown*/
 	//GF_LOG(GF_LOG_DEBUG, GF_LOG_CORE, ("[Core] Waiting for thread to start\n"));
 	gf_sema_wait(t->_signal);
 	gf_sema_del(t->_signal);
@@ -390,11 +391,11 @@ GF_Err gf_th_run(GF_Thread *t, u32 (*Run)(void *param), void *param)
 static void Thread_Stop(GF_Thread *t, Bool Destroy)
 {
 	if (gf_th_status(t) == GF_THREAD_STATUS_RUN) {
-		if (Destroy){
+		if (Destroy) {
 			t->threadH->Terminate(0);
 			t->threadH = NULL;
 		}
-		else{
+		else {
 			t->threadH->Suspend();
 		}
 	}
@@ -412,9 +413,9 @@ void gf_th_del(GF_Thread *t)
 {
 	Thread_Stop(t, 0);
 #ifndef GPAC_DISABLE_LOG
-	free(t->log_name);
+	gf_free(t->log_name);
 #endif
-	free(t);
+	gf_free(t);
 }
 
 
@@ -464,22 +465,22 @@ struct __tag_mutex
 GF_EXPORT
 GF_Mutex *gf_mx_new(const char *name)
 {
-	GF_Mutex *tmp = (GF_Mutex *)malloc(sizeof(GF_Mutex));
+	GF_Mutex *tmp = (GF_Mutex *)gf_malloc(sizeof(GF_Mutex));
 	if (!tmp) return NULL;
 	memset(tmp, 0, sizeof(GF_Mutex));
-	
+
 	tmp->hMutex = new RMutex();
-	if( tmp->hMutex->CreateLocal() != KErrNone){
-		free(tmp);
+	if( tmp->hMutex->CreateLocal() != KErrNone) {
+		gf_free(tmp);
 		return NULL;
 	}
 #ifndef GPAC_DISABLE_LOG
 	if (name) {
-		tmp->log_name = strdup(name);
+		tmp->log_name = gf_strdup(name);
 	} else {
 		char szN[20];
 		sprintf(szN, "0x%08x", (u32) tmp);
-		tmp->log_name = strdup(szN);
+		tmp->log_name = gf_strdup(szN);
 	}
 #endif
 	return tmp;
@@ -489,7 +490,7 @@ GF_EXPORT
 void gf_mx_del(GF_Mutex *mx)
 {
 	mx->hMutex->Close();
-	free(mx);
+	gf_free(mx);
 }
 
 GF_EXPORT
@@ -562,12 +563,12 @@ struct __tag_semaphore
 GF_EXPORT
 GF_Semaphore *gf_sema_new(u32 MaxCount, u32 InitCount)
 {
-	GF_Semaphore *tmp = (GF_Semaphore *) malloc(sizeof(GF_Semaphore));
+	GF_Semaphore *tmp = (GF_Semaphore *) gf_malloc(sizeof(GF_Semaphore));
 
 	if (!tmp) return NULL;
 	tmp->hSemaphore = new RSemaphore();
 	if (!tmp->hSemaphore) {
-		free(tmp);
+		gf_free(tmp);
 		return NULL;
 	}
 	TBuf<32>	semaName;
@@ -581,21 +582,15 @@ GF_EXPORT
 void gf_sema_del(GF_Semaphore *sm)
 {
 	sm->hSemaphore->Close();
-	free(sm);
+	gf_free(sm);
 }
 
 GF_EXPORT
-u32 gf_sema_notify(GF_Semaphore *sm, u32 NbRelease)
+Bool gf_sema_notify(GF_Semaphore *sm, u32 NbRelease)
 {
-	u32 prevCount;
-	if (!sm) return 0;
+    if (!sm) return GF_FALSE;
 	sm->hSemaphore->Signal(NbRelease);
-#ifdef __SERIES60_3X__
-	prevCount = 0;
-#else
-	prevCount = sm->hSemaphore->Count();
-#endif
-	return (u32) prevCount;
+	return GF_TRUE;
 }
 
 GF_EXPORT
@@ -637,7 +632,7 @@ void gf_sys_close()
 	}
 }
 
-#if GPAC_MEMORY_TRACKING
+#ifdef GPAC_MEMORY_TRACKING
 extern size_t gpac_allocated_memory;
 #endif
 
@@ -670,7 +665,7 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	if (rti->sampling_instant + refresh_time_ms > now) return 0;
 	rti->sampling_period_duration = now - rti->sampling_instant;
 	rti->sampling_instant = now;
-	
+
 	if (cur_th.Process(cur_process) != KErrNone) {
 		return 0;
 	}
@@ -693,18 +688,18 @@ Bool gf_sys_get_rti(u32 refresh_time_ms, GF_SystemRTInfo *rti, u32 flags)
 	rti->process_cpu_usage = 100*rti->process_cpu_time_diff / rti->sampling_period_duration;
 	if (rti->process_cpu_usage > 100) rti->process_cpu_usage = 100;
 
-	HAL::Get(HALData::EMemoryRAM, ram);		
-	HAL::Get(HALData::EMemoryRAMFree, ram_free);		
+	HAL::Get(HALData::EMemoryRAM, ram);
+	HAL::Get(HALData::EMemoryRAMFree, ram_free);
 	rti->physical_memory = ram;
 	rti->physical_memory_avail = ram_free;
-#if GPAC_MEMORY_TRACKING
+#ifdef GPAC_MEMORY_TRACKING
 	rti->gpac_memory = gpac_allocated_memory;
 #endif
 	return 1;
 }
 
 GF_EXPORT
-Bool gf_sys_get_battery_state(Bool *onBattery, u32 *state, u32*level) 
+Bool gf_sys_get_battery_state(Bool *onBattery, u32 *state, u32*level)
 {
 	return 1;
 }
@@ -713,27 +708,26 @@ Bool gf_sys_get_battery_state(Bool *onBattery, u32 *state, u32*level)
 /*delete all interfaces loaded on object*/
 void gf_modules_free_module(ModuleInstance *inst)
 {
-	void *objinterface;
 	while (gf_list_count(inst->interfaces)) {
-		objinterface = gf_list_get(inst->interfaces, 0);
+		void *objinterface = gf_list_get(inst->interfaces, 0);
 		gf_list_rem(inst->interfaces, 0);
 		inst->destroy_func(objinterface);
 	}
-	if (inst->lib_handle){
+	if (inst->lib_handle) {
 		RLibrary* pLibrary = (RLibrary *) inst->lib_handle;
 		pLibrary->Close();
 	}
 	gf_list_del(inst->interfaces);
-	free(inst);
+	gf_free(inst);
 }
 
 Bool gf_modules_load_library(ModuleInstance *inst)
 {
-	const TUid KGPACModuleUid={0x10000080};
+	const TUid KGPACModuleUid= {0x10000080};
 	char s_path[GF_MAX_PATH];
 	HBufC *path;
-    TInt e;
-	
+	TInt e;
+
 	if (inst->lib_handle) return 1;
 
 	sprintf(s_path, "%s%c%s", inst->plugman->dir, GF_PATH_SEPARATOR, inst->szName);
@@ -757,10 +751,10 @@ Bool gf_modules_load_library(ModuleInstance *inst)
 		delete pLibrary;
 		goto err_exit;
 	}
-	inst->query_func = (QueryInterface) pLibrary->Lookup(1);
+	inst->query_func = (QueryInterfaces) pLibrary->Lookup(1);
 	inst->load_func = (LoadInterface) pLibrary->Lookup(2);
 	inst->destroy_func = (ShutdownInterface) pLibrary->Lookup(3);
-	
+
 	if ((inst->query_func==NULL) || (inst->load_func==NULL) || (inst->destroy_func==NULL) ) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[core] Library %s has invalid interfaces", inst->szName));
 		pLibrary->Close();
@@ -774,7 +768,7 @@ Bool gf_modules_load_library(ModuleInstance *inst)
 	return 1;
 
 err_exit:
-	gf_cfg_set_key(inst->plugman->cfg, "SymbianDLLs", inst->szName, "no"); 
+	gf_cfg_set_key(inst->plugman->cfg, "SymbianDLLs", inst->szName, "no");
 	return 0;
 }
 
@@ -785,7 +779,7 @@ void gf_modules_unload_library(ModuleInstance *inst)
 	RLibrary* pLibrary = (RLibrary *) inst->lib_handle;
 	pLibrary->Close();
 	delete pLibrary;
-	
+
 	inst->lib_handle = NULL;
 	inst->load_func = NULL;
 	inst->destroy_func = NULL;
@@ -794,13 +788,12 @@ void gf_modules_unload_library(ModuleInstance *inst)
 }
 
 
-static Bool enum_modules(void *cbck, char *item_name, char *item_path)
+static Bool enum_modules(void *cbck, char *item_name, char *item_path, GF_FileEnumInfo *file_info)
 {
 	ModuleInstance *inst;
 
 	GF_ModuleManager *pm = (GF_ModuleManager *) cbck;
 
-	if (strstr(item_name, "nposmozilla")) return 0;
 	if (strncmp(item_name, "gm_", 3)) return 0;
 	if (gf_module_is_loaded(pm, item_name) ) return 0;
 
@@ -824,11 +817,11 @@ u32 gf_modules_refresh(GF_ModuleManager *pm)
 	gf_enum_directory((char*)pm->dir, 0, enum_modules, pm, ".dll");
 #else
 	u32 i, mod_count;
-	
+
 	mod_count = gf_cfg_get_key_count(pm->cfg, "SymbianDLLs");
 	for (i=0; i<mod_count; i++) {
 		const char *mod = gf_cfg_get_key_name(pm->cfg, "SymbianDLLs", i);
-		if (stricmp(gf_cfg_get_key(pm->cfg, "SymbianDLLs", mod), "yes")) continue; 
+		if (stricmp(gf_cfg_get_key(pm->cfg, "SymbianDLLs", mod), "yes")) continue;
 		enum_modules(pm, (char*)mod, NULL);
 	}
 #endif
